@@ -41,6 +41,9 @@ import urllib
 # import io
 import re
 from typing import Any, Union
+
+from i2 import Sig
+
 from http2py.constants import (
     BINARY_CONTENT_TYPE,
     FORM_CONTENT_TYPE,
@@ -55,6 +58,7 @@ from http2py.default_configs import (
     default_binary_output_trans,
 )
 
+# TODO: Do we really need this anymore?
 with I2mintModuleNotFoundErrorNiceMessage():
     from i2.util import inject_method, imdict
     from i2.signatures import set_signature_of_func, KO
@@ -121,6 +125,8 @@ def mk_param_spec_from_arg_schema(arg, required=False):
     return spec_dict
 
 
+# TODO: Deprecated output_trans in favor of egress, for consistency with other i2imint?
+# TODO: Use method_spec object that has more schema, signature, and validation.
 def mk_request_function(
     method_spec, *, function_kind='method', dispatch=request, verify_cert=True
 ):
@@ -175,7 +181,7 @@ def mk_request_function(
     body_arg_names = _ensure_list(method_spec.get('body_arg_names', []))
     arg_specs = method_spec.get('arg_specs', [])
     formatted_arg_specs = [mk_param_spec_from_arg_schema(arg) for arg in arg_specs]
-    func_args = path_arg_names + query_arg_names + body_arg_names
+    func_args = list(path_arg_names) + list(query_arg_names) + list(body_arg_names)
 
     debug = method_spec.pop('debug', None)
     if 'debug' in method_spec:
@@ -212,7 +218,7 @@ def mk_request_function(
         url = None
         if 'url_template' in method_spec:
             url_template = method_spec['url_template']
-            # Check if the url template already have parameters, add them otherwise
+            # Check if the url template already has parameters, add them otherwise
             if query_arg_names and not re.search('^.*\?((.*=.*)(&?))+$', url_template):
                 url_arg_parts = [f'{x}={{{x}}}' for x in query_arg_names if x in kwargs]
                 url_args = '&'.join(url_arg_parts)
@@ -227,20 +233,27 @@ def mk_request_function(
                 k: urllib.parse.quote(v, safe='') if type(v) is str else v
                 for k, v in param_kwargs.items()
             }
-            url = url_template.format(**encoded_kwargs)
+            # TODO: Added this _kwargs_for_url_template to get kwargs, because
+            #  encoded_kwargs alone stopped working when I added a signature to
+            #  Py2Request methods.
+            #  But really, don't know what I'm doing, becaues this request_func
+            #  function is a mess and should be revised
+            _kwargs_for_url_template = dict(kwargs, **encoded_kwargs)
+            url = url_template.format(**_kwargs_for_url_template)
         elif 'url' in method_spec:
             url = method_spec['url']
 
         json_kwargs = {k: v for k, v in kwargs.items() if is_jsonable(v)}
         binary_kwargs = {k: v for k, v in kwargs.items() if k not in json_kwargs}
-        if content_type in (JSON_CONTENT_TYPE, RAW_CONTENT_TYPE):
-            if binary_kwargs:
-                raise RuntimeError(
-                    f'Some of the inputs are not JSON-serializable, impossible to send them over an {content_type} request.'
-                )
-            _request_kwargs['data'] = json.dumps(json_kwargs)
-        elif content_type == BINARY_CONTENT_TYPE:
-            _request_kwargs['data'] = pickle.dumps(kwargs)
+        if method != 'GET':  # problem needing this arose when adding method sigs
+            if content_type in (JSON_CONTENT_TYPE, RAW_CONTENT_TYPE):
+                if binary_kwargs:
+                    raise RuntimeError(
+                        f'Some of the inputs are not JSON-serializable, impossible to send them over an {content_type} request.'
+                    )
+                _request_kwargs['data'] = json.dumps(json_kwargs)
+            elif content_type == BINARY_CONTENT_TYPE:
+                _request_kwargs['data'] = pickle.dumps(kwargs)
         elif content_type == FORM_CONTENT_TYPE:
             fields = json.dumps(json_kwargs).encode('utf-8')
             binaries = {k: (f'{k}.bin', v) for k, v in binary_kwargs.items()}
@@ -296,7 +309,7 @@ str_formatter = string.Formatter()
 
 
 class Py2Request(object):
-    """ Make a class that has methods that offer a python interface to web requests """
+    """Make a class that has methods that offer a python interface to web requests"""
 
     def __init__(
         self,
@@ -333,7 +346,7 @@ class Py2Request(object):
         ...     print(r.text)
         >>> def dict_of_json(r):
         ...     return json.loads(r.content)
-        >>> tokenizer = re.compile('\w+').findall
+        >>> tokenizer = re.compile('\\w+').findall
         >>> # Defining the specs
         >>> method_specs = {
         ...     'google_google': {
@@ -369,46 +382,101 @@ class Py2Request(object):
         """
         self._method_specs = dict(method_specs or {}, **kwargs)
         self._dflt_method_func_from_method_spec = method_func_from_method_spec
-        self._process_method_specs()
+        self._method_specs = self._process_method_specs()
 
         for method_name, method_spec in self._method_specs.items():
             self._inject_method(method_name, method_spec, method_func_from_method_spec)
 
     def _process_method_specs(self):
-        if self._dflt_method_func_from_method_spec == mk_request_function:
-            for method_name, method_spec in self._method_specs.items():
-                if 'args' not in method_spec and 'url_template' in method_spec:
-                    method_spec['args'] = list(
-                        filter(
-                            bool,
-                            (
-                                x[1]
-                                for x in str_formatter.parse(
-                                    method_spec['url_template']
-                                )
-                            ),
-                        )
-                    )
+        if self._dflt_method_func_from_method_spec == mk_request_function:  # TODO: WTF!
+
+            def gen():
+                for method_name, method_spec in self._method_specs.items():
+                    if isinstance(method_spec, str):
+                        # TODO: feat: add signature control in string method_spec?
+                        method_spec = {'method': 'GET', 'url_template': method_spec}
+                    if 'url_template' in method_spec:
+                        if 'args' not in method_spec:
+                            method_spec['args'] = extract_format_names(
+                                method_spec['url_template']
+                            )
+                        if 'query_arg_names' not in method_spec:
+                            method_spec['query_arg_names'] = extract_format_names(
+                                method_spec['url_template']
+                            )
+                    yield method_name, method_spec
+
+            return dict(gen())
+        else:
+            return self._method_specs
 
     def _inject_method(
         self, method_name, method_spec, method_func_from_method_spec=None
     ):
         method_wrap = None
         if not callable(method_spec):
-            method_spec = dict(**method_spec)
-            args = []
-            if isinstance(method_spec, dict):
-                args = method_spec.get('args', [])
-            method_wrap = method_spec.pop('method_wrap', None)
-            if method_func_from_method_spec is None:
-                method_func_from_method_spec = self._dflt_method_func_from_method_spec
-            method_spec = method_func_from_method_spec(method_spec)
+            method_spec, method_wrap = _mk_method_func_and_wrap(
+                method_spec,
+                method_func_from_method_spec or self._dflt_method_func_from_method_spec,
+            )
         inject_method(self, method_spec, method_name)
 
         if method_wrap is not None:
             setattr(self, method_name, method_wrap(getattr(self, method_name)))
 
 
+def _mk_method_func_and_wrap(method_spec, method_func_from_method_spec):
+    method_spec = dict(**method_spec)
+    method_wrap = method_spec.pop('method_wrap', None)
+    url_template = method_spec.get('url_template', None)
+    method_func = method_func_from_method_spec(method_spec)
+    if url_template is not None:
+        # Add a signature to the method_func
+        if 'signature' in method_spec:
+            sig = Sig(method_spec['signature'])
+        else:
+            arg_names = extract_format_names(url_template)
+            # TODO: What about args? difference between args and query_arg_names?
+            pk_names = method_spec.get('query_arg_names', ())
+            sig = _mk_signature_from_names(arg_names, pk_names)
+        method_func = sig(method_func)
+    return method_func, method_wrap
+
+
+def _mk_signature_from_names(arg_names, pk_names):
+    assert set(pk_names) <= set(
+        arg_names
+    ), 'The query_arg_names must be a subset of the names in the url_template'
+    ko_names = _difference_conserving_order(arg_names, pk_names)
+    if ko_names:
+        ko_names_str = '*, ' + ', '.join(ko_names)
+    else:
+        ko_names_str = ''
+    sig = Sig(f"(self, {', '.join(pk_names)}{ko_names_str})")
+    return sig
+
+
+def _difference_conserving_order(a, b):
+    """
+    Returns a list with the elements of a that are not in b, but in the same order as a.
+    """
+    return [x for x in a if x not in b]
+
+
+def extract_format_names(string_template):
+    """
+    Extracts the names of the fields in a string template, returning a list of
+    strings.
+
+    >>> string_template = "Hello, {name}! You are {age} years old."
+    >>> extract_format_names(string_template)
+    ('name', 'age')
+    """
+    NAME_IDX = 1
+    parsed_template = str_formatter.parse(string_template)
+    return tuple(filter(bool, (x[NAME_IDX] for x in parsed_template)))
+
+
 class UrlMethodSpecsMaker:
     """
     Utility to help in making templated method_specs dicts to be used to define a Py2Request object.
@@ -545,10 +613,10 @@ def mk_method_spec_from_openapi_method_spec(
         )
         for argname, details in body_properties.items():
             body_arg_names.append(argname)
-            argtype = details['type']
             # TODO: fully support typed dict and typed iterable types
             arg_spec = {'name': argname}
-            pytype = pytype_for_oatype[argtype]
+            argtype = details.get('type', None)
+            pytype = pytype_for_oatype.get(argtype, None)
             if pytype:
                 arg_spec['type'] = pytype
             if 'default' in details:
